@@ -19,8 +19,10 @@
  */
 
 #include <mediastreamer2/msfilter.h>
+#include <mediastreamer2/msjava.h>
 #include <mediastreamer2/msticker.h>
 #include <mediastreamer2/mssndcard.h>
+#include <mediastreamer2/devices.h>
 
 #include "AudioTrack.h"
 #include "AudioRecord.h"
@@ -28,7 +30,6 @@
 
 #include <jni.h>
 
-#include "audiofilters/devices.h"
 #include "hardware_echo_canceller.h"
 
 #define NATIVE_USE_HARDWARE_RATE 1
@@ -40,8 +41,8 @@ using namespace::fake_android;
 static const float audio_buf_ms=0.01;
 
 static MSSndCard * android_snd_card_new(SoundDeviceDescription *d);
-static MSFilter * ms_android_snd_read_new(void);
-static MSFilter * ms_android_snd_write_new(void);
+static MSFilter * ms_android_snd_read_new(MSFactory* factory);
+static MSFilter * ms_android_snd_write_new(MSFactory* factory);
 static Library *libmedia=0;
 static Library *libutils=0;
 
@@ -205,13 +206,13 @@ struct AndroidSndWriteData{
 };
 
 static MSFilter *android_snd_card_create_reader(MSSndCard *card){
-	MSFilter *f=ms_android_snd_read_new();
+	MSFilter *f=ms_android_snd_read_new(ms_snd_card_get_factory(card));
 	(static_cast<AndroidSndReadData*>(f->data))->setCard(card);
 	return f;
 }
 
 static MSFilter *android_snd_card_create_writer(MSSndCard *card){
-	MSFilter *f=ms_android_snd_write_new();
+	MSFilter *f=ms_android_snd_write_new(ms_snd_card_get_factory(card));
 	(static_cast<AndroidSndWriteData*>(f->data))->setCard((AndroidNativeSndCardData*)card->data);
 	return f;
 }
@@ -267,6 +268,7 @@ static void android_snd_card_detect(MSSndCardManager *m){
 	if (audio_record_loaded && audio_track_loaded && audio_system_loaded && string8_loaded && refbase_loaded && !(d->flags & DEVICE_HAS_UNSTANDARD_LIBMEDIA)){
 		ms_message("Native android sound support available.");
 		MSSndCard *card=android_snd_card_new(d);
+		ms_snd_card_set_manager(m,card);
 		ms_snd_card_manager_add_card(m,card);
 		return;
 	}
@@ -347,11 +349,11 @@ static void android_snd_read_cb(int event, void* user, void *p_info){
 		 *  1) understand why AudioRecord thread doesn't detach.
 		 *  2) disable logs just for this thread (using a TLS)
 		 */
-		int loglevel=ortp_get_log_level_mask();
-		ortp_set_log_level_mask(ORTP_ERROR|ORTP_FATAL);
+		int loglevel=ortp_get_log_level_mask(ORTP_LOG_DOMAIN);
+		ortp_set_log_level_mask(NULL, ORTP_ERROR|ORTP_FATAL);
 		ad->mTickerSynchronizer = ms_ticker_synchronizer_new();
 		ms_ticker_set_time_func(obj->ticker,(uint64_t (*)(void*))ms_ticker_synchronizer_get_corrected_time, ad->mTickerSynchronizer);
-		ortp_set_log_level_mask(loglevel);
+		ortp_set_log_level_mask(ORTP_LOG_DOMAIN, loglevel);
 	}
 	if (event==AudioRecord::EVENT_MORE_DATA){
 		AudioRecord::Buffer info;
@@ -561,8 +563,8 @@ MSFilterDesc android_snd_read_desc={
 	android_snd_read_methods
 };
 
-static MSFilter * ms_android_snd_read_new(){
-	MSFilter *f=ms_filter_new_from_desc(&android_snd_read_desc);
+static MSFilter * ms_android_snd_read_new(MSFactory *factory){
+	MSFilter *f=ms_factory_create_filter_from_desc(factory, &android_snd_read_desc);
 	return f;
 }
 
@@ -632,7 +634,7 @@ static void android_snd_write_cb(int event, void *user, void * p_info){
 			ms_bufferizer_skip_bytes(&ad->bf, avail - (ask * 2));
 		}
 		if (avail != 0) {
-			if ((ad->minBufferFilling == -1)) {
+			if (ad->minBufferFilling == -1) {
 				ad->minBufferFilling = avail;
 			} else if (avail < ad->minBufferFilling) {
 				ad->minBufferFilling = avail;
@@ -817,8 +819,8 @@ MSFilterDesc android_snd_write_desc={
 	android_snd_write_methods
 };
 
-static MSFilter * ms_android_snd_write_new(void){
-	MSFilter *f=ms_filter_new_from_desc(&android_snd_write_desc);
+static MSFilter * ms_android_snd_write_new(MSFactory* factory){
+	MSFilter *f=ms_factory_create_filter_from_desc(factory, &android_snd_write_desc);
 	return f;
 }
 
