@@ -23,6 +23,7 @@
 #include "mediastreamer2/msfilter.h"
 #include "mediastreamer2/msticker.h"
 #include "mediastreamer2/msjava.h"
+#include "mediastreamer2/devices.h"
 #include <jni.h>
 
 #include <sys/time.h>
@@ -31,7 +32,6 @@
 #include "mediastreamer2/zrtp.h"
 #include <cpu-features.h>
 
-#include "audiofilters/devices.h"
 #include "hardware_echo_canceller.h"
 
 static const float sndwrite_flush_threshold=0.020;	//ms
@@ -364,7 +364,8 @@ static void* msandroid_read_cb(msandroid_sound_read_data* d) {
 static void sound_read_setup(MSFilter *f){
 	ms_debug("andsnd_read_preprocess");
 	msandroid_sound_read_data *d=(msandroid_sound_read_data*)f->data;
-	jmethodID constructor_id=0;
+	jmethodID constructor_id=0, methodID = 0;
+	int audio_record_state=0;
 	jmethodID min_buff_size_id;
 	//jmethodID set_notification_period;
 	int rc;
@@ -422,12 +423,25 @@ static void sound_read_setup(MSFilter *f){
 			,2/*  ENCODING_PCM_16BIT */
 			,d->buff_size);
 
-
-	d->audio_record = jni_env->NewGlobalRef(d->audio_record);
-	if (d->audio_record == 0) {
-		ms_error("cannot instantiate AudioRecord");
+	//Check the state of the AudioRecord (uninitialized = 1
+	methodID = jni_env->GetMethodID(d->audio_record_class,"getState", "()I");
+	if (methodID == 0) {
+		ms_error("cannot find AudioRecord getState() method");
 		return;
 	}
+	audio_record_state = jni_env->CallIntMethod(d->audio_record, methodID);
+
+	if(audio_record_state == 1) {
+		d->audio_record = jni_env->NewGlobalRef(d->audio_record);
+		if (d->audio_record == 0) {
+			ms_error("cannot instantiate AudioRecord");
+			return;
+		}
+	} else {
+		d->audio_record = NULL;
+		ms_error("AudioRecord is not initialized properly. It may be caused by RECORD_AUDIO permission not granted");
+	}
+
 	d->min_avail=-1;
 	d->read_samples=0;
 	d->ticker_synchronizer = ms_ticker_synchronizer_new();
@@ -436,10 +450,12 @@ static void sound_read_setup(MSFilter *f){
 	d->framesize=(d->outgran_ms*d->rate)/1000;
 	d->started=true;
 	// start reader thread
-	rc = ms_thread_create(&d->thread_id, 0, (void*(*)(void*))msandroid_read_cb, d);
-	if (rc){
-		ms_error("cannot create read thread return code  is [%i]", rc);
-		d->started=false;
+	if(d->audio_record) {
+		rc = ms_thread_create(&d->thread_id, 0, (void*(*)(void*))msandroid_read_cb, d);
+		if (rc){
+			ms_error("cannot create read thread return code  is [%i]", rc);
+			d->started=false;
+        }
 	}
 }
 
@@ -485,13 +501,11 @@ static void sound_read_postprocess(MSFilter *f){
 		goto end;
 	}
 
-	if (d->aec) {
-		delete_hardware_echo_canceller(jni_env, d->aec);
-		d->aec = NULL;
-	}
-
 	d->started = false;
-	if (d->thread_id !=0) ms_thread_join(d->thread_id,0);
+	if (d->thread_id !=0 ){
+		ms_thread_join(d->thread_id,0);
+		d->thread_id = 0;
+	}
 
 	if (d->audio_record) {
 		jni_env->CallVoidMethod(d->audio_record,stop_id);
@@ -504,6 +518,10 @@ static void sound_read_postprocess(MSFilter *f){
 		}
 		jni_env->CallVoidMethod(d->audio_record,release_id);
 	}
+	if (d->aec) {
+		delete_hardware_echo_canceller(jni_env, d->aec);
+		d->aec = NULL;
+	}
 	goto end;
 	end: {
 		if (d->audio_record) jni_env->DeleteGlobalRef(d->audio_record);
@@ -569,7 +587,7 @@ static MSFilterDesc msandroid_sound_read_desc={
 
 MSFilter *msandroid_sound_read_new(MSSndCard *card){
 	ms_debug("msandroid_sound_read_new");
-	MSFilter *f=ms_filter_new_from_desc(&msandroid_sound_read_desc);
+	MSFilter *f=ms_factory_create_filter_from_desc(ms_snd_card_get_factory(card), &msandroid_sound_read_desc);
 	msandroid_sound_read_data *data=new msandroid_sound_read_data();
 	data->builtin_aec = card->capabilities & MS_SND_CARD_CAP_BUILTIN_ECHO_CANCELLER;
 	if (card->data != NULL) {
@@ -584,8 +602,6 @@ MSFilter *msandroid_sound_read_new(MSSndCard *card){
 	return f;
 }
 
-MS_FILTER_DESC_EXPORT(msandroid_sound_read_desc)
-
 /***********************************write filter********************/
 static int set_write_rate(MSFilter *f, void *arg){
 #ifndef USE_HARDWARE_RATE
@@ -669,10 +685,12 @@ static void* msandroid_write_cb(msandroid_sound_write_data* d) {
 	int check_point_size=3*(float)d->rate*(float)d->nchannels*2.0; /*3 seconds*/
 	int nwrites=0;
 
-	set_high_prio();
 	int buff_size = d->write_chunk_size;
+	uint8_t tmpBuff[buff_size];
 	JNIEnv *jni_env = ms_get_jni_env();
 
+	set_high_prio();
+
 	// int write  (byte[] audioData, int offsetInBytes, int sizeInBytes)
 	write_id = jni_env->GetMethodID(d->audio_track_class,"write", "([BII)I");
 	if(write_id==0) {
@@ -685,7 +703,6 @@ static void* msandroid_write_cb(msandroid_sound_write_data* d) {
 		goto end;
 	}
 	write_buff = jni_env->NewByteArray(buff_size);
-	uint8_t tmpBuff[buff_size];
 
 	//start playing
 	jni_env->CallVoidMethod(d->audio_track,play_id);
@@ -820,7 +837,10 @@ void msandroid_sound_write_postprocess(MSFilter *f){
 	ms_mutex_lock(&d->mutex);
 	ms_cond_signal(&d->cond);
 	ms_mutex_unlock(&d->mutex);
-	ms_thread_join(d->thread_id,0);
+	if (d->thread_id != 0){
+		ms_thread_join(d->thread_id, NULL);
+		d->thread_id = 0;
+	}
 	// flush
 	flush_id = jni_env->GetMethodID(d->audio_track_class,"flush", "()V");
 	if(flush_id==0) {
@@ -895,7 +915,7 @@ static MSFilterDesc msandroid_sound_write_desc={
 
 MSFilter *msandroid_sound_write_new(MSSndCard *card){
 	ms_debug("msandroid_sound_write_new");
-	MSFilter *f=ms_filter_new_from_desc(&msandroid_sound_write_desc);
+	MSFilter *f=ms_factory_create_filter_from_desc(ms_snd_card_get_factory(card), &msandroid_sound_write_desc);
 	msandroid_sound_write_data *data = new msandroid_sound_write_data();
 	if (card->data != NULL) {
 		SoundDeviceDescription *d = (SoundDeviceDescription *)card->data;
@@ -910,8 +930,6 @@ MSFilter *msandroid_sound_write_new(MSSndCard *card){
 }
 
 
-MS_FILTER_DESC_EXPORT(msandroid_sound_write_desc)
-
 
 
 /******* Hack for Galaxy S ***********/
