@@ -50,10 +50,10 @@ then try incrementing the number of periods*/
 having sound quality trouble:*/
 /*#define EPIPE_BUGFIX 1*/
 
-static MSSndCard * alsa_card_new(int id, const char *pcmbasename);
+static MSSndCard * alsa_card_new(const char *id, const char *pcmbasename);
 static MSSndCard *alsa_card_duplicate(MSSndCard *obj);
-static MSFilter * ms_alsa_read_new(const char *dev);
-static MSFilter * ms_alsa_write_new(const char *dev);
+static MSFilter * ms_alsa_read_new(MSFactory *factory, const char *dev);
+static MSFilter * ms_alsa_write_new(MSFactory *factory, const char *dev);
 
 
 struct _AlsaData{
@@ -604,14 +604,14 @@ static void alsa_card_set_source(MSSndCard *obj,MSSndCardCapture source)
 static MSFilter *alsa_card_create_reader(MSSndCard *card)
 {
 	AlsaData *ad=(AlsaData*)card->data;
-	MSFilter *f=ms_alsa_read_new(ad->pcmdev);
+	MSFilter *f=ms_alsa_read_new(ms_snd_card_get_factory(card), ad->pcmdev);
 	return f;
 }
 
 static MSFilter *alsa_card_create_writer(MSSndCard *card)
 {
 	AlsaData *ad=(AlsaData*)card->data;
-	MSFilter *f=ms_alsa_write_new(ad->pcmdev);
+	MSFilter *f=ms_alsa_write_new(ms_snd_card_get_factory(card), ad->pcmdev);
 	return f;
 }
 
@@ -620,7 +620,7 @@ void alsa_error_log_handler(const char *file, int line, const char *function, in
 	char * format = ms_strdup_printf("also error in %s:%d - %s", file, line, fmt);
 	va_list args;
 	va_start (args, fmt);
-	ortp_logv(ORTP_MESSAGE, format, args);
+	ortp_logv(ORTP_LOG_DOMAIN, ORTP_MESSAGE, format, args);
 	va_end (args);
 	ms_free(format);
 }
@@ -640,34 +640,101 @@ static void alsa_card_uninit(MSSndCard *obj){
 }
 
 static void alsa_card_detect(MSSndCardManager *m){
-	int i;
+	int i,j,k;
 	void **hints=NULL;
+	int hint_device_count=0;
 	int device_count=0;
-	int old_device_count=0; /*previous alsa pcm naming convention*/
-	const char *pcmname = "sysdefault";
+	const char *pcm_base_name = "plug";
+	bool_t found_duplicate;
+	const int MAX_NUM_DEVICE_ID = 100;
+	const int MAX_PLUG_NAME_CHARS = 50;
+	char *plug_names[MAX_NUM_DEVICE_ID]; 
+	char *card_names[MAX_NUM_DEVICE_ID], *device_names[MAX_NUM_DEVICE_ID];
+	char *unique_card_names[MAX_NUM_DEVICE_ID], *unique_device_names[MAX_NUM_DEVICE_ID];
+
+	/* Get list of devices from alsa device hints */
 
-	/*get the number of PCM device availables*/
 	if (snd_device_name_hint(-1, "pcm", &hints)==0){
-		int i;
 		for(i=0; hints[i]!=NULL; ++i){
-			char *name = snd_device_name_get_hint(hints[i],"NAME");
-			if (name && strstr(name,"sysdefault") == name) device_count++;
-			if (name && strstr(name,"default") == name) old_device_count++;
-			if (name) free(name);
+			char *hint = snd_device_name_get_hint(hints[i],"NAME");
+			char *hint_ptr = hint;
+			if (hint != NULL) {
+				char *device_name = strsep(&hint, ":");
+				if (device_name){
+					char *card_hint = strsep(&hint, ",");
+					if (card_hint){
+						if (strcmp(strsep(&card_hint, "="), "CARD")==0){
+							char *card_name = card_hint;
+							card_names[hint_device_count] = ms_strdup(card_name);
+							device_names[hint_device_count] = ms_strdup(device_name);
+							hint_device_count++;
+						}
+					}
+				}
+				free(hint_ptr);
+			}
 		}
 		snd_device_name_free_hint(hints);
 	}
-	if (device_count == 0) {
-		device_count = old_device_count;
-		pcmname = "default";
+
+	/* Produce unique device_names[] */
+
+	for (j=0; j<hint_device_count; j++){
+		found_duplicate = FALSE;
+		if (j == 0){
+			unique_card_names[0] = card_names[0];
+			unique_device_names[0] = device_names[0];
+			plug_names[device_count] = (char*) ms_new(char, MAX_PLUG_NAME_CHARS);
+                        sprintf(plug_names[0],"%s:%s", device_names[0], card_names[0]);
+
+			device_count++;
+			continue;
+		}
+		for (k=0; k<device_count; k++){
+
+			if ( (strcmp(card_names[j], unique_card_names[k])==0) && (strcmp(device_names[j], unique_device_names[k])==0) ){
+				found_duplicate = TRUE;
+				break;
+			}
+		}
+		if ( !found_duplicate ){
+			unique_card_names[device_count] = card_names[j];
+			unique_device_names[device_count] = device_names[j];
+
+			plug_names[device_count] = (char*) ms_new(char,MAX_PLUG_NAME_CHARS);
+			sprintf(plug_names[device_count],"%s:%s", device_names[j], card_names[j]);
+
+			device_count++;
+		}
 	}
+
+	/* Add unique devices to card manager */
+
 	for (i=-1;i<device_count;i++){
-		MSSndCard *card=alsa_card_new(i, pcmname);
+		MSSndCard *card=NULL;
+		if (i==-1){
+			card=alsa_card_new("default", pcm_base_name);
+		}
+		else {
+			// Currently, only 'default' devices are supported.
+			if (strcmp(unique_device_names[i], "default")==0) {
+				card=alsa_card_new(plug_names[i], pcm_base_name);
+			}
+		}
+
 		if (card!=NULL)
+		{
 			ms_snd_card_manager_add_card(m,card);
-		else break;
+		}
 	}
+
 	atexit((void(*)(void))snd_config_update_free_global);
+
+	for (i=0; i<device_count; i++) {
+		ms_free(plug_names[i]);
+		ms_free(device_names[i]);
+		ms_free(card_names[i]);
+	}
 }
 
 MSSndCardDesc alsa_card_desc={
@@ -752,22 +819,21 @@ static unsigned int get_card_capabilities(const char *devname, char **card_name,
 	return ret;
 }
 
-static MSSndCard * alsa_card_new(int id, const char *pcmbasename){
+static MSSndCard * alsa_card_new(const char *id, const char *pcmbasename){
 	MSSndCard * obj;
 	AlsaData *ad;
 	unsigned int hw_index=0;
 
 	obj=ms_snd_card_new(&alsa_card_desc);
 	ad=(AlsaData*)obj->data;
-	if (id==-1) {
+	if (strcmp(id,"default")==0) {
 		/* the default pcm device */
 		obj->name=ms_strdup("default device");
 		ad->pcmdev=ms_strdup("default");
 		ad->mixdev=ms_strdup("default");
 	}else{
 		snd_mixer_t *mixer;
-		if (id == 0) ad->pcmdev = ms_strdup(pcmbasename);
-		else ad->pcmdev=ms_strdup_printf("%s:%i",pcmbasename, id);
+		ad->pcmdev=ms_strdup_printf("%s:%s",pcmbasename, id);
 		obj->capabilities = get_card_capabilities(ad->pcmdev, &obj->name, &hw_index);
 		if (obj->capabilities == 0){
 			ms_snd_card_destroy(obj);
@@ -1052,8 +1118,8 @@ MSFilterDesc alsa_read_desc={
 	.methods=alsa_read_methods
 };
 
-static MSFilter * ms_alsa_read_new(const char *dev){
-	MSFilter *f=ms_filter_new_from_desc(&alsa_read_desc);
+static MSFilter * ms_alsa_read_new(MSFactory *factory, const char *dev){
+	MSFilter *f=ms_factory_create_filter_from_desc(factory, &alsa_read_desc);
 	AlsaReadData *ad=(AlsaReadData*)f->data;
 	ad->pcmdev=ms_strdup(dev);
 	return f;
@@ -1164,8 +1230,8 @@ MSFilterDesc alsa_write_desc={
 };
 
 
-static MSFilter * ms_alsa_write_new(const char *dev){
-	MSFilter *f=ms_filter_new_from_desc(&alsa_write_desc);
+static MSFilter * ms_alsa_write_new(MSFactory *factory, const char *dev){
+	MSFilter *f = ms_factory_create_filter_from_desc(factory, &alsa_write_desc);
 	AlsaWriteData *ad=(AlsaWriteData*)f->data;
 	ad->pcmdev=ms_strdup(dev);
 	return f;
