@@ -26,12 +26,16 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 
 static int rec_close(MSFilter *f, void *arg);
+static void write_wav_header(int fd, int rate, int nchannels, int size);
 
 typedef struct RecState{
 	int fd;
 	int rate;
 	int nchannels;
 	int size;
+	int max_size;
+	char *mime;
+	bool_t swap;
 	MSRecorderState state;
 } RecState;
 
@@ -41,10 +45,23 @@ static void rec_init(MSFilter *f){
 	s->rate=8000;
 	s->nchannels = 1;
 	s->size=0;
+	s->max_size=0;
 	s->state=MSRecorderClosed;
+	s->mime = "pcm";
+	s->swap = FALSE;
 	f->data=s;
 }
 
+static void swap_bytes(unsigned char *bytes, int len){
+	int i;
+	unsigned char tmp;
+	for(i=0;i<len;i+=2){
+		tmp=bytes[i];
+		bytes[i]=bytes[i+1];
+		bytes[i+1]=tmp;
+	}
+}
+
 static void rec_process(MSFilter *f){
 	RecState *s=(RecState*)f->data;
 	mblk_t *m;
@@ -55,12 +72,28 @@ static void rec_process(MSFilter *f){
 		if (s->state==MSRecorderRunning){
 			while(it!=NULL){
 				int len=(int)(it->b_wptr-it->b_rptr);
+				int max_size_reached = 0;
+				if (s->max_size!=0 && s->size+len > s->max_size) {
+					len = s->max_size - s->size;
+					max_size_reached = 1;
+				}
+				if (s->swap) swap_bytes(it->b_wptr,len);
 				if ((err=write(s->fd,it->b_rptr,len))!=len){
 					if (err<0)
 						ms_warning("MSFileRec: fail to write %i bytes: %s",len,strerror(errno));
 				}
 				it=it->b_cont;
 				s->size+=len;
+				if (max_size_reached) {
+					ms_warning("MSFileRec: Maximum size (%d) has been reached. closing file.",s->max_size);
+					s->state=MSRecorderClosed;
+					if (s->fd!=-1){
+						write_wav_header(s->fd, s->rate, s->nchannels, s->size);
+						close(s->fd);
+						s->fd=-1;
+					}
+					ms_filter_notify_no_arg(f,MS_RECORDER_MAX_SIZE_REACHED);
+				}
 			}
 		}
 		ms_mutex_unlock(&f->lock);
@@ -188,21 +221,61 @@ static int rec_set_sr(MSFilter *f, void *arg){
 	return 0;
 }
 
+static int rec_get_sr(MSFilter *f, void *arg){
+	RecState *d=(RecState*)f->data;
+	int *sample_rate = (int *)arg;
+	*sample_rate = d->rate;
+	return 0;
+}
+
 static int rec_set_nchannels(MSFilter *f, void *arg) {
 	RecState *s = (RecState *)f->data;
 	s->nchannels = *(int *)arg;
 	return 0;
 }
 
+static int rec_get_nchannels(MSFilter *f, void *arg){
+	RecState *d=(RecState*)f->data;
+	int *nchannels = (int *)arg;
+	*nchannels = d->nchannels;
+	return 0;
+}
+
 static void rec_uninit(MSFilter *f){
 	RecState *s=(RecState*)f->data;
 	if (s->fd!=-1) rec_close(f,NULL);
 	ms_free(s);
 }
 
+static int rec_get_fmtp(MSFilter *f, void *arg){
+	RecState *d=(RecState*)f->data;
+	MSPinFormat *pinfmt = (MSPinFormat*)arg;
+	if (pinfmt->pin == 0) pinfmt->fmt = ms_factory_get_audio_format(f->factory, d->mime, d->rate, d->nchannels, NULL);
+	return 0;
+}
+
+static int rec_set_fmtp(MSFilter *f, void *arg){
+	RecState *d=(RecState*)f->data;
+	MSPinFormat *pinfmt = (MSPinFormat*)arg;
+	ms_filter_lock(f);
+	d->rate = pinfmt->fmt->rate;
+	d->nchannels = pinfmt->fmt->nchannels;
+	d->mime = pinfmt->fmt->encoding;
+	ms_filter_unlock(f);
+	return 0;
+}
+
+static int rec_set_max_size(MSFilter *f, void *arg) {
+	RecState *d=(RecState*)f->data;
+	d->max_size = *((int *) arg);
+	return 0;
+}
+
 static MSFilterMethod rec_methods[]={
 	{	MS_FILTER_SET_SAMPLE_RATE,	rec_set_sr	},
 	{	MS_FILTER_SET_NCHANNELS	,	rec_set_nchannels	},
+	{	MS_FILTER_GET_SAMPLE_RATE,	rec_get_sr	},
+	{	MS_FILTER_GET_NCHANNELS	,	rec_get_nchannels	},
 	{	MS_FILE_REC_OPEN	,	rec_open	},
 	{	MS_FILE_REC_START	,	rec_start	},
 	{	MS_FILE_REC_STOP	,	rec_stop	},
@@ -212,6 +285,9 @@ static MSFilterMethod rec_methods[]={
 	{	MS_RECORDER_PAUSE	,	rec_stop	},
 	{	MS_RECORDER_CLOSE	,	rec_close	},
 	{	MS_RECORDER_GET_STATE	,	rec_get_state	},
+	{ 	MS_FILTER_GET_OUTPUT_FMT, 	rec_get_fmtp },
+	{ 	MS_FILTER_SET_OUTPUT_FMT, 	rec_set_fmtp },
+	{	MS_RECORDER_SET_MAX_SIZE,	rec_set_max_size },
 	{	0			,	NULL		}
 };
 
