@@ -48,8 +48,6 @@ static const char * audio_unit_format_error (OSStatus error) {
 		case kAudioUnitErr_NoConnection: return "kAudioUnitErr_NoConnection";
 		case kAudioUnitErr_FailedInitialization: return "kAudioUnitErr_FailedInitialization";
 		case kAudioUnitErr_TooManyFramesToProcess: return "kAudioUnitErr_TooManyFramesToProcess";
-		case kAudioUnitErr_IllegalInstrument: return "kAudioUnitErr_IllegalInstrument";
-		case kAudioUnitErr_InstrumentTypeNotFound: return "kAudioUnitErr_InstrumentTypeNotFound";
 		case kAudioUnitErr_InvalidFile: return "kAudioUnitErr_InvalidFile";
 		case kAudioUnitErr_UnknownFileType: return "kAudioUnitErr_UnknownFileType";
 		case kAudioUnitErr_FileNotSpecified: return "kAudioUnitErr_FileNotSpecified";
@@ -63,7 +61,7 @@ static const char * audio_unit_format_error (OSStatus error) {
 		case kAudioUnitErr_Initialized: return "kAudioUnitErr_Initialized";
 		case kAudioUnitErr_InvalidOfflineRender: return "kAudioUnitErr_InvalidOfflineRender";
 		case kAudioUnitErr_Unauthorized: return "kAudioUnitErr_Unauthorized";
-		default: return "unkown error";
+		default: return "unknown error";
 	}
 
 }
@@ -98,10 +96,10 @@ static const char *audio_session_format_error(OSStatus error)
  }
 
 #define check_au_session_result(au,method) \
-if (au!=0) ms_error("AudioSession error for %s: ret=%s (%li) (%s:%d)",method, audio_session_format_error(au), au, __FILE__, __LINE__ )
+if (au!=0) ms_error("AudioSession error for %s: ret=%s (%li) (%s:%d)",method, audio_session_format_error(au), (long)au, __FILE__, __LINE__ )
 
 #define check_au_unit_result(au,method) \
-if (au!=0) ms_error("AudioUnit error for %s: ret=%s (%li) (%s:%d)",method, audio_unit_format_error(au), au, __FILE__, __LINE__ )
+if (au!=0) ms_error("AudioUnit error for %s: ret=%s (%li) (%s:%d)",method, audio_unit_format_error(au), (long)au, __FILE__, __LINE__ )
 
 
 #define check_session_call(call)   do { OSStatus res = (call); check_au_session_result(res, #call); } while(0)
@@ -165,6 +163,7 @@ struct au_filter_write_data{
 	ms_mutex_t	mutex;
 	MSBufferizer	*bufferizer;
 	unsigned int n_lost_frame;
+	bool first_frame_wrote;
 
 };
 
@@ -202,7 +201,7 @@ static void create_io_unit (AudioUnit* au, MSSndCard* sndcard) {
 	AudioComponentDescription au_description;
 	AudioComponent foundComponent;
 
-	bool_t noVoiceProc = (*au && strcasecmp(sndcard->name, AU_CARD_NOVOICEPROC) == 0);
+	bool_t noVoiceProc = (strcasecmp(sndcard->name, AU_CARD_NOVOICEPROC) == 0);
 	OSType subtype = noVoiceProc ? kAudioUnitSubType_RemoteIO : kAudioUnitSubType_VoiceProcessingIO;
 
 	au_description.componentType          = kAudioUnitType_Output;
@@ -384,12 +383,17 @@ static OSStatus au_write_cb (
 
 	if (d!=NULL){
 		ms_mutex_lock(&d->mutex);
+		if (!d->first_frame_wrote) {
+			ms_bufferizer_flush(d->bufferizer); /*to avoid keeping delay from first start which can be around 100ms becasue io unit takes time to start*/
+			d->first_frame_wrote=true;
+		}
 		if(ms_bufferizer_get_avail(d->bufferizer) >= inNumberFrames*d->base.card->bits/8) {
 			ms_bufferizer_read(d->bufferizer, ioData->mBuffers[0].mData, inNumberFrames*d->base.card->bits/8);
-			if (ms_bufferizer_get_avail(d->bufferizer) >10*inNumberFrames*d->base.card->bits/8) {
-				ms_debug("we are late, bufferizer sise is %i bytes in framezize is %lu bytes"
-						,ms_bufferizer_get_avail(d->bufferizer)
-						,inNumberFrames*d->base.card->bits/8);
+			/*basic algo,  can be enhanced with a more advanced bufferizer computing average value*/
+			if (ms_bufferizer_get_avail(d->bufferizer) > card->rate* (card->nchannels * card->bits / 8)/5 ) {
+				ms_warning("we are at least 200ms late, bufferizer sise is %li bytes in framezize is %li bytes"
+						,(long)ms_bufferizer_get_avail(d->bufferizer)
+						,(long)inNumberFrames*d->base.card->bits/8);
 				ms_bufferizer_flush(d->bufferizer);
 			}
 			ms_mutex_unlock(&d->mutex);
@@ -412,6 +416,7 @@ static OSStatus au_write_cb (
 /****************config**************/
 static void configure_audio_session (au_card_t* d,uint64_t time) {
 	UInt32 audioCategory;
+	UInt32 audioMode;
 	UInt32 audioCategorySize=sizeof(audioCategory);
 	bool_t changed;
 
@@ -422,7 +427,7 @@ static void configure_audio_session (au_card_t* d,uint64_t time) {
 			check_session_call(AudioSessionGetProperty(kAudioSessionProperty_AudioCategory,&audioCategorySize,&audioCategory));
 
 			changed=(audioCategory!=kAudioSessionCategory_AmbientSound && d->is_ringer)
-				||(audioCategory!=kAudioSessionCategory_PlayAndRecord && !d->is_ringer);
+			||(audioCategory!=kAudioSessionCategory_PlayAndRecord && !d->is_ringer);
 		}
 
 		if (!d->audio_session_configured || changed) {
@@ -430,13 +435,16 @@ static void configure_audio_session (au_card_t* d,uint64_t time) {
 
 			if (d->is_ringer && kCFCoreFoundationVersionNumber > kCFCoreFoundationVersionNumber10_6 /*I.E is >=OS4*/) {
 				audioCategory= kAudioSessionCategory_AmbientSound;
+				audioMode = kAudioSessionMode_Default;
 				ms_message("Configuring audio session for playback");
 			} else {
 				audioCategory = kAudioSessionCategory_PlayAndRecord;
+				audioMode = kAudioSessionMode_VoiceChat;
 				ms_message("Configuring audio session for playback/record");
 			}
 
 			check_audiounit_call(AudioSessionSetProperty(kAudioSessionProperty_AudioCategory, sizeof(audioCategory), &audioCategory));
+			check_audiounit_call(AudioSessionSetProperty(kAudioSessionProperty_Mode, sizeof(audioMode), &audioMode));
 
 		}else{
 			ms_message("Audio session already correctly configured.");
@@ -477,7 +485,7 @@ static bool_t  start_audio_unit (au_filter_base_t* d,uint64_t time) {
 									  , &quality
 									  , &qualitySize));
 
-		ms_message("I/O unit latency [%f], quality [%li]",delay,quality);
+		ms_message("I/O unit latency [%f], quality [%u]",delay,(unsigned)quality);
 		Float32 hwoutputlatency;
 		UInt32 hwoutputlatencySize=sizeof(hwoutputlatency);
 
@@ -523,7 +531,7 @@ static void destroy_audio_unit (au_card_t* d) {
 		AudioComponentInstanceDispose (d->io_unit);
 		d->io_unit=NULL;
 		if (!d->is_fast) {
-			check_session_call( AudioSessionSetActiveWithFlags(false, kAudioSessionSetActiveFlag_NotifyOthersOnDeactivation) );
+			AudioSessionSetActiveWithFlags(false, kAudioSessionSetActiveFlag_NotifyOthersOnDeactivation);
 		}
 		ms_message("AudioUnit destroyed");
 	}
@@ -861,6 +869,8 @@ static void au_read_uninit(MSFilter *f) {
 	check_unused(card);
 
 	ms_mutex_destroy(&d->mutex);
+
+	flushq(&d->rq,0);
 	ms_free(d);
 }
 
@@ -911,7 +921,7 @@ MSFilterDesc au_write_desc={
 static MSFilter *ms_au_read_new(MSSndCard *mscard){
 	ms_debug("ms_au_read_new");
 	au_card_t* card=(au_card_t*)(mscard->data);
-	MSFilter *f=ms_filter_new_from_desc(&au_read_desc);
+	MSFilter *f=ms_factory_create_filter_from_desc(ms_snd_card_get_factory(mscard), &au_read_desc);
 	au_filter_read_data_t *d=ms_new0(au_filter_read_data_t,1);
 	qinit(&d->rq);
 	d->readTimeStamp.mSampleTime=-1;
@@ -925,7 +935,7 @@ static MSFilter *ms_au_read_new(MSSndCard *mscard){
 static MSFilter *ms_au_write_new(MSSndCard *mscard){
 	ms_debug("ms_au_write_new");
 	au_card_t* card=(au_card_t*)(mscard->data);
-	MSFilter *f=ms_filter_new_from_desc(&au_write_desc);
+	MSFilter *f=ms_factory_create_filter_from_desc(ms_snd_card_get_factory(mscard), &au_write_desc);
 	au_filter_write_data_t *d=ms_new0(au_filter_write_data_t,1);
 	d->bufferizer= ms_bufferizer_new();
 	ms_mutex_init(&d->mutex,NULL);
