@@ -40,8 +40,9 @@ struct _PAData{
 
 typedef struct _PAData PAData;
 
-static pa_context *context=NULL;
-static pa_threaded_mainloop *pa_loop=NULL;
+static int the_pa_ref = 0;
+static pa_context *the_pa_context=NULL;
+static pa_threaded_mainloop *the_pa_loop=NULL;
 static const int targeted_latency = 20;/*ms*/
 
 static void context_state_notify_cb(pa_context *ctx, void *userdata){
@@ -71,42 +72,47 @@ static void context_state_notify_cb(pa_context *ctx, void *userdata){
 		break;
 	}
 	ms_message("New PulseAudio context state: %s",sname);
-	pa_threaded_mainloop_signal(pa_loop, FALSE);
+	pa_threaded_mainloop_signal(the_pa_loop, FALSE);
 }
 
 static bool_t wait_for_context_state(pa_context_state_t success_state, pa_context_state_t failure_state){
 	pa_context_state_t state;
-	pa_threaded_mainloop_lock(pa_loop);
-	state = pa_context_get_state(context);
+	pa_threaded_mainloop_lock(the_pa_loop);
+	state = pa_context_get_state(the_pa_context);
 	while(state != success_state && state != failure_state) {
-		pa_threaded_mainloop_wait(pa_loop);
-		state = pa_context_get_state(context);
+		pa_threaded_mainloop_wait(the_pa_loop);
+		state = pa_context_get_state(the_pa_context);
 	}
-	pa_threaded_mainloop_unlock(pa_loop);
+	pa_threaded_mainloop_unlock(the_pa_loop);
 	return state == success_state;
 }
 
-static void init_pulse_context(){
-	if (context==NULL){
-		pa_loop=pa_threaded_mainloop_new();
-		context=pa_context_new(pa_threaded_mainloop_get_api(pa_loop),NULL);
-		pa_context_set_state_callback(context,context_state_notify_cb,NULL);
-		pa_context_connect(context, NULL, 0, NULL);
-		pa_threaded_mainloop_start(pa_loop);
+
+static void init_pulse_context(void){
+	if (the_pa_ref == 0){
+		the_pa_loop=pa_threaded_mainloop_new();
+		the_pa_context=pa_context_new(pa_threaded_mainloop_get_api(the_pa_loop),NULL);
+		pa_context_set_state_callback(the_pa_context,context_state_notify_cb,NULL);
+		pa_context_connect(the_pa_context, NULL, 0, NULL);
+		pa_threaded_mainloop_start(the_pa_loop);
+	}
+	the_pa_ref++;
+}
+
+static void uninit_pulse_context(void){
+	the_pa_ref--;
+	if (the_pa_ref == 0){
+		pa_context_disconnect(the_pa_context);
+		pa_context_unref(the_pa_context);
+		pa_threaded_mainloop_stop(the_pa_loop);
+		pa_threaded_mainloop_free(the_pa_loop);
+		the_pa_context = NULL;
+		the_pa_loop = NULL;
 	}
 }
 
-static void uninit_pulse_context(){
-	pa_context_disconnect(context);
-	pa_context_unref(context);
-	pa_threaded_mainloop_stop(pa_loop);
-	pa_threaded_mainloop_free(pa_loop);
-	context = NULL;
-	pa_loop = NULL;
-}
-
 static void pulse_card_unload(MSSndCardManager *m) {
-	if(context) uninit_pulse_context();
+	uninit_pulse_context();
 }
 
 static void pulse_card_detect(MSSndCardManager *m);
@@ -136,42 +142,42 @@ MSSndCardDesc pulse_card_desc={
 };
 
 void pa_sinklist_cb(pa_context *c, const pa_sink_info *l, int eol, void *userdata) {
-	MSList **pa_devicelist = userdata;
+	bctbx_list_t **pa_devicelist = userdata;
 	pa_device_t *pa_device;
 
 	/* when eol is set to a positive number : end of the list */
 	if (eol > 0) {
-		return;
+		goto end;
 	}
 
 	pa_device = ms_malloc0(sizeof(pa_device_t));
 	strncpy(pa_device->name, l->name, PA_STRING_SIZE-1);
 	strncpy(pa_device->description, l->description, PA_STRING_SIZE-1);
 
-	*pa_devicelist = ms_list_append(*pa_devicelist, pa_device);
-	
-	pa_threaded_mainloop_signal(pa_loop, FALSE);
+	*pa_devicelist = bctbx_list_append(*pa_devicelist, pa_device);
+end:
+	pa_threaded_mainloop_signal(the_pa_loop, FALSE);
 }
 
 void pa_sourcelist_cb(pa_context *c, const pa_source_info *l, int eol, void *userdata) {
-	MSList **pa_devicelist = userdata;
+	bctbx_list_t **pa_devicelist = userdata;
 	pa_device_t *pa_device;
 
 	if (eol > 0) {
-		return;
+		goto end;
 	}
 
 	if (l->monitor_of_sink!=PA_INVALID_INDEX) { /* ignore monitors */
-		return;
+		goto end;
 	}
 	
 	pa_device = ms_malloc0(sizeof(pa_device_t));
 	strncpy(pa_device->name, l->name, PA_STRING_SIZE -1);
 	strncpy(pa_device->description, l->description, PA_STRING_SIZE -1);
 
-	*pa_devicelist = ms_list_append(*pa_devicelist, pa_device);
-	
-	pa_threaded_mainloop_signal(pa_loop, FALSE);
+	*pa_devicelist = bctbx_list_append(*pa_devicelist, pa_device);
+end:
+	pa_threaded_mainloop_signal(the_pa_loop, FALSE);
 }
 
 /* Add cards to card manager, list contains sink only and bidirectionnal cards */
@@ -221,13 +227,13 @@ int pulse_card_compare(pa_device_t *sink, pa_device_t *source) {
 	return strncmp(sink->description, source->description, 512);
 }
 
-static void pulse_card_merge_lists(pa_device_t *pa_device, MSList **pa_source_list) {
-	MSList *sourceCard = ms_list_find_custom(*pa_source_list, (MSCompareFunc)pulse_card_compare, pa_device); 
+static void pulse_card_merge_lists(pa_device_t *pa_device, bctbx_list_t **pa_source_list) {
+	bctbx_list_t *sourceCard = bctbx_list_find_custom(*pa_source_list, (bctbx_compare_func)pulse_card_compare, pa_device); 
 	if (sourceCard!= NULL) {
 		pa_device_t *sourceCard_data = (pa_device_t *)sourceCard->data;
 		pa_device->bidirectionnal = 1;
 		strncpy(pa_device->source_name,sourceCard_data->name, PA_STRING_SIZE -1);
-		*pa_source_list = ms_list_remove(*pa_source_list, sourceCard->data);
+		*pa_source_list = bctbx_list_remove(*pa_source_list, sourceCard->data);
 		ms_free(sourceCard_data);	
 	}
 }
@@ -240,8 +246,8 @@ static void pulse_card_merge_lists(pa_device_t *pa_device, MSList **pa_source_li
  **/
 static void pulse_card_detect(MSSndCardManager *m){
 	pa_operation *pa_op;
-	MSList *pa_sink_list=NULL;
-	MSList *pa_source_list=NULL;
+	bctbx_list_t *pa_sink_list=NULL;
+	bctbx_list_t *pa_source_list=NULL;
 	
 	/* connect to pulse server */
 	init_pulse_context();
@@ -250,37 +256,37 @@ static void pulse_card_detect(MSSndCardManager *m){
 		return;
 	}
 
-	pa_threaded_mainloop_lock(pa_loop);
+	pa_threaded_mainloop_lock(the_pa_loop);
 	
 	/* retrieve all available sinks */
-	pa_op = pa_context_get_sink_info_list(context, pa_sinklist_cb, &pa_sink_list);
+	pa_op = pa_context_get_sink_info_list(the_pa_context, pa_sinklist_cb, &pa_sink_list);
 
 	/* wait for the operation to complete */
 	while (pa_operation_get_state(pa_op) != PA_OPERATION_DONE) {
-		pa_threaded_mainloop_wait(pa_loop);
+		pa_threaded_mainloop_wait(the_pa_loop);
 	}
 	pa_operation_unref(pa_op);
 
 	/* retrieve all available sources, monitors are ignored */
-	pa_op = pa_context_get_source_info_list(context, pa_sourcelist_cb, &pa_source_list);
+	pa_op = pa_context_get_source_info_list(the_pa_context, pa_sourcelist_cb, &pa_source_list);
 
 	/* wait for the operation to complete */
 	while (pa_operation_get_state(pa_op) != PA_OPERATION_DONE) {
-		pa_threaded_mainloop_wait(pa_loop);
+		pa_threaded_mainloop_wait(the_pa_loop);
 	}
 	pa_operation_unref(pa_op);
 
-	pa_threaded_mainloop_unlock(pa_loop);
+	pa_threaded_mainloop_unlock(the_pa_loop);
 	
 	/* merge source list into sink list for dual capabilities cards */
-	ms_list_for_each2(pa_sink_list, (MSIterate2Func)pulse_card_merge_lists, &pa_source_list);
+	bctbx_list_for_each2(pa_sink_list, (MSIterate2Func)pulse_card_merge_lists, &pa_source_list);
 
 	/* create sink and souce cards */
-	ms_list_for_each2(pa_sink_list, (MSIterate2Func)pulse_card_sink_create, m);
-	ms_list_for_each2(pa_source_list, (MSIterate2Func)pulse_card_source_create, m);
+	bctbx_list_for_each2(pa_sink_list, (MSIterate2Func)pulse_card_sink_create, m);
+	bctbx_list_for_each2(pa_source_list, (MSIterate2Func)pulse_card_source_create, m);
 
-	ms_list_free_with_data(pa_sink_list, ms_free);
-	ms_list_free_with_data(pa_source_list, ms_free);
+	bctbx_list_free_with_data(pa_sink_list, ms_free);
+	bctbx_list_free_with_data(pa_source_list, ms_free);
 }
 
 typedef enum _StreamType {
@@ -309,15 +315,15 @@ static void stream_disconnect(Stream *s);
 static void stream_state_notify_cb(pa_stream *p, void *userData) {
 	Stream *ctx = (Stream *)userData;
 	ctx->state = pa_stream_get_state(p);
-	pa_threaded_mainloop_signal(pa_loop, 0);
+	pa_threaded_mainloop_signal(the_pa_loop, 0);
 }
 
 static bool_t stream_wait_for_state(Stream *ctx, pa_stream_state_t successState, pa_stream_state_t failureState) {
-	pa_threaded_mainloop_lock(pa_loop);
+	pa_threaded_mainloop_lock(the_pa_loop);
 	while(ctx->state != successState && ctx->state != failureState) {
-		pa_threaded_mainloop_wait(pa_loop);
+		pa_threaded_mainloop_wait(the_pa_loop);
 	}
-	pa_threaded_mainloop_unlock(pa_loop);
+	pa_threaded_mainloop_unlock(the_pa_loop);
 	return ctx->state == successState;
 }
 
@@ -349,11 +355,17 @@ static void stream_free(Stream *s) {
 }
 
 static size_t stream_play(Stream *s, size_t nbytes) {
-	if (nbytes == 0) return 0;
+	size_t avail;
 
-	if (ms_bufferizer_get_avail(&s->bufferizer) >= nbytes){
+	if (nbytes == 0)
+		return 0;
+
+	avail = ms_bufferizer_get_avail(&s->bufferizer);
+	if (avail > 0) {
 		uint8_t *data;
 		int buffer_size;
+		if (nbytes > avail)
+			nbytes = avail;
 		data = ms_new(uint8_t, nbytes);
 		ms_mutex_lock(&s->mutex);
 		ms_bufferizer_read(&s->bufferizer, data, nbytes);
@@ -412,17 +424,17 @@ static bool_t stream_connect(Stream *s) {
 	}
 	
 	
-	if (context==NULL) {
+	if (the_pa_context==NULL) {
 		ms_error("No PulseAudio context");
 		return FALSE;
 	}
-	s->stream=pa_stream_new(context,"phone",&s->sampleSpec,NULL);
+	s->stream=pa_stream_new(the_pa_context,"phone",&s->sampleSpec,NULL);
 	if (s->stream==NULL){
 		ms_error("fails to create PulseAudio stream");
 		return FALSE;
 	}
 	pa_stream_set_state_callback(s->stream, stream_state_notify_cb, s);
-	pa_threaded_mainloop_lock(pa_loop);
+	pa_threaded_mainloop_lock(the_pa_loop);
 	if(s->type == STREAM_TYPE_PLAYBACK) {
 		pa_stream_set_write_callback(s->stream, stream_write_request_cb, s);
 		pa_stream_set_overflow_callback(s->stream, stream_buffer_overflow_notification, s);
@@ -435,7 +447,7 @@ static bool_t stream_connect(Stream *s) {
 	} else {
 		err=pa_stream_connect_record(s->stream,s->dev,&attr, PA_STREAM_ADJUST_LATENCY);
 	}
-	pa_threaded_mainloop_unlock(pa_loop);
+	pa_threaded_mainloop_unlock(the_pa_loop);
 	if(err < 0 || !stream_wait_for_state(s, PA_STREAM_READY, PA_STREAM_FAILED)) {
 		ms_error("Fails to connect pulseaudio stream. err=%d", err);
 		pa_stream_unref(s->stream);
@@ -453,9 +465,9 @@ static bool_t stream_connect(Stream *s) {
 static void stream_disconnect(Stream *s) {
 	int err;
 	if (s->stream) {
-		pa_threaded_mainloop_lock(pa_loop);
+		pa_threaded_mainloop_lock(the_pa_loop);
 		err = pa_stream_disconnect(s->stream);
-		pa_threaded_mainloop_unlock(pa_loop);
+		pa_threaded_mainloop_unlock(the_pa_loop);
 		if(err!=0 || !stream_wait_for_state(s, PA_STREAM_TERMINATED, PA_STREAM_FAILED)) {
 			ms_error("pa_stream_disconnect() failed. err=%d", err);
 		}
@@ -469,7 +481,7 @@ static void stream_disconnect(Stream *s) {
 
 static void stream_set_volume_cb(pa_context *c, int success, void *user_data) {
 	*(int *)user_data = success;
-	pa_threaded_mainloop_signal(pa_loop, FALSE);
+	pa_threaded_mainloop_signal(the_pa_loop, FALSE);
 }
 
 static bool_t stream_set_volume(Stream *s, double volume) {
@@ -490,16 +502,16 @@ static bool_t stream_set_volume(Stream *s, double volume) {
 	idx = pa_stream_get_index(s->stream);
 	pa_cvolume_init(&cvolume);
 	pa_cvolume_set(&cvolume, s->sampleSpec.channels, scale_to_volume(volume));
-	pa_threaded_mainloop_lock(pa_loop);
+	pa_threaded_mainloop_lock(the_pa_loop);
 	if(s->type == STREAM_TYPE_PLAYBACK) {
-		op = pa_context_set_sink_input_volume(context, idx, &cvolume, stream_set_volume_cb, &success);
+		op = pa_context_set_sink_input_volume(the_pa_context, idx, &cvolume, stream_set_volume_cb, &success);
 	} else {
-		op = pa_context_set_source_output_volume(context, idx, &cvolume, stream_set_volume_cb, &success);
+		op = pa_context_set_source_output_volume(the_pa_context, idx, &cvolume, stream_set_volume_cb, &success);
 	}
 	while(pa_operation_get_state(op) == PA_OPERATION_RUNNING) {
-		pa_threaded_mainloop_wait(pa_loop);
+		pa_threaded_mainloop_wait(the_pa_loop);
 	}
-	pa_threaded_mainloop_unlock(pa_loop);
+	pa_threaded_mainloop_unlock(the_pa_loop);
 	pa_operation_unref(op);
 	return success;
 }
@@ -508,14 +520,14 @@ static void stream_get_source_volume_cb(pa_context *c, const pa_source_output_in
 	if(i) {
 		*(double *)user_data = volume_to_scale(pa_cvolume_avg(&i->volume));
 	}
-	pa_threaded_mainloop_signal(pa_loop, FALSE);
+	pa_threaded_mainloop_signal(the_pa_loop, FALSE);
 }
 
 static void stream_get_sink_volume_cb(pa_context *c, const pa_sink_input_info *i, int eol, void *user_data) {
 	if(i) {
 		*(double *)user_data = volume_to_scale(pa_cvolume_avg(&i->volume));
 	}
-	pa_threaded_mainloop_signal(pa_loop, FALSE);
+	pa_threaded_mainloop_signal(the_pa_loop, FALSE);
 }
 
 static bool_t stream_get_volume(Stream *s, double *volume) {
@@ -528,16 +540,16 @@ static bool_t stream_get_volume(Stream *s, double *volume) {
 	}
 	idx = pa_stream_get_index(s->stream);
 	*volume = -1.0;
-	pa_threaded_mainloop_lock(pa_loop);
+	pa_threaded_mainloop_lock(the_pa_loop);
 	if(s->type == STREAM_TYPE_PLAYBACK) {
-		op = pa_context_get_sink_input_info(context, idx, stream_get_sink_volume_cb, volume);
+		op = pa_context_get_sink_input_info(the_pa_context, idx, stream_get_sink_volume_cb, volume);
 	} else {
-		op = pa_context_get_source_output_info(context, idx, stream_get_source_volume_cb, volume);
+		op = pa_context_get_source_output_info(the_pa_context, idx, stream_get_source_volume_cb, volume);
 	}
 	while(pa_operation_get_state(op) == PA_OPERATION_RUNNING) {
-		pa_threaded_mainloop_wait(pa_loop);
+		pa_threaded_mainloop_wait(the_pa_loop);
 	}
-	pa_threaded_mainloop_unlock(pa_loop);
+	pa_threaded_mainloop_unlock(the_pa_loop);
 	pa_operation_unref(op);
 	return TRUE;
 }
@@ -568,7 +580,7 @@ static void pulse_read_process(MSFilter *f){
 		ms_error("Record stream not connected");
 		return;
 	}
-	pa_threaded_mainloop_lock(pa_loop);
+	pa_threaded_mainloop_lock(the_pa_loop);
 	while(pa_stream_readable_size(s->stream) > 0) {
 		if(pa_stream_peek(s->stream, &buffer, &nbytes) >= 0) {
 			if(buffer != NULL) {
@@ -585,7 +597,7 @@ static void pulse_read_process(MSFilter *f){
 			break;
 		}
 	}
-	pa_threaded_mainloop_unlock(pa_loop);
+	pa_threaded_mainloop_unlock(the_pa_loop);
 }
 
 static void pulse_read_postprocess(MSFilter *f) {
@@ -712,10 +724,10 @@ static void pulse_write_process(MSFilter *f){
 		ms_bufferizer_put_from_queue(&s->bufferizer,f->inputs[0]);
 		ms_mutex_unlock(&s->mutex);
 		
-		pa_threaded_mainloop_lock(pa_loop);
+		pa_threaded_mainloop_lock(the_pa_loop);
 		nwritable = pa_stream_writable_size(s->stream);
 		stream_play(s, nwritable);
-		pa_threaded_mainloop_unlock(pa_loop);
+		pa_threaded_mainloop_unlock(the_pa_loop);
 		
 		if (s->last_stats == (uint64_t)-1) {
 			s->last_stats = f->ticker->time;
@@ -724,9 +736,9 @@ static void pulse_write_process(MSFilter *f){
 			int is_negative;
 			int err;
 			s->last_stats = f->ticker->time;
-			pa_threaded_mainloop_lock(pa_loop);
+			pa_threaded_mainloop_lock(the_pa_loop);
 			err = pa_stream_get_latency(s->stream, &latency, &is_negative);
-			pa_threaded_mainloop_unlock(pa_loop);
+			pa_threaded_mainloop_unlock(the_pa_loop);
 			if (err == 0 && !is_negative) {
 				ms_message("pulseaudio: latency is equal to %d ms", (int)(latency/1000L));
 			}
@@ -792,7 +804,7 @@ static MSFilterDesc pulse_write_desc={
 
 static MSFilter *pulse_card_create_reader(MSSndCard *card) {
 	PAData *card_data = (PAData *)card->data;
-	MSFilter *f=ms_filter_new_from_desc (&pulse_read_desc);
+	MSFilter *f=ms_factory_create_filter_from_desc(ms_snd_card_get_factory(card), &pulse_read_desc);
 	Stream *s = (Stream *)f->data;
 	s->dev = ms_strdup(card_data->pa_id_source);  /* add pulse audio card id to connect the stream to the correct card */
 	return f;
@@ -800,7 +812,7 @@ static MSFilter *pulse_card_create_reader(MSSndCard *card) {
 
 static MSFilter *pulse_card_create_writer(MSSndCard *card) {
 	PAData *card_data = (PAData *)card->data;
-	MSFilter *f=ms_filter_new_from_desc (&pulse_write_desc);
+	MSFilter *f=ms_factory_create_filter_from_desc(ms_snd_card_get_factory(card), &pulse_write_desc);
 	Stream *s = (Stream *)f->data;
 	s->dev = ms_strdup(card_data->pa_id_sink); /* add pulse audio card id to connect the stream to the correct card */
 	return f;
