@@ -27,6 +27,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "mediastreamer2_tester_private.h"
 #include "qosanalyzer.h"
 #include <math.h>
+#include <sys/stat.h>
 
 static RtpProfile rtp_profile;
 
@@ -34,6 +35,7 @@ static RtpProfile rtp_profile;
 #define SPEEX_PAYLOAD_TYPE 122
 #define SILK16_PAYLOAD_TYPE  123
 #define PCMA8_PAYLOAD_TYPE 8
+#define BV16_PAYLOAD_TYPE 127
 #define H263_PAYLOAD_TYPE 34
 #define H264_PAYLOAD_TYPE 102
 #define VP8_PAYLOAD_TYPE 103
@@ -41,9 +43,15 @@ static RtpProfile rtp_profile;
 #define EDGE_BW 10
 #define THIRDGENERATION_BW 200
 
+static MSFactory *_factory = NULL;
 static int tester_before_all(void) {
-	ms_init();
-	ms_filter_enable_statistics(TRUE);
+
+	_factory = ms_factory_new();
+	ms_factory_init_voip(_factory);
+	ms_factory_init_plugins(_factory);
+
+	//ms_filter_enable_statistics(TRUE);
+	ms_factory_enable_statistics(_factory, TRUE);
 	ortp_init();
 	rtp_profile_set_payload (&rtp_profile,0,&payload_type_pcmu8000);
 	rtp_profile_set_payload (&rtp_profile,OPUS_PAYLOAD_TYPE,&payload_type_opus);
@@ -59,13 +67,16 @@ static int tester_before_all(void) {
 
 static int tester_after_all(void) {
 	ortp_exit();
-	ms_exit();
+
+	ms_factory_destroy(_factory);
 	rtp_profile_clear_all(&rtp_profile);
 	return 0;
 }
 
 #define HELLO_16K_1S_FILE  "sounds/hello16000-1s.wav"
-#define RECORDED_16K_1S_FILE  "sounds/recorded_hello16000-1s.wav"
+#define RECORDED_16K_1S_FILE  "recorded_hello16000-1s.wav"
+//#define RECORDED_16K_1S_FILE  "recorded_hello16000-1sbv16.wav"
+#define RECORDED_16K_1S_NO_PLC_FILE  "withoutplc_recorded_hello16000-1sbv16.wav"
 
 typedef struct _stream_manager_t {
 	MSFormatType type;
@@ -96,10 +107,10 @@ stream_manager_t * stream_manager_new(MSFormatType type) {
 	mgr->user_data = 0;
 
 	if (mgr->type==MSAudio){
-		mgr->audio_stream=audio_stream_new (mgr->local_rtp, mgr->local_rtcp,FALSE);
+		mgr->audio_stream=audio_stream_new (_factory, mgr->local_rtp, mgr->local_rtcp,FALSE);
 	}else{
 #if VIDEO_ENABLED
-		mgr->video_stream=video_stream_new (mgr->local_rtp, mgr->local_rtcp,FALSE);
+		mgr->video_stream=video_stream_new (_factory, mgr->local_rtp, mgr->local_rtcp,FALSE);
 #else
 		ms_fatal("Unsupported stream type [%s]",ms_format_type_to_string(mgr->type));
 #endif
@@ -110,12 +121,12 @@ stream_manager_t * stream_manager_new(MSFormatType type) {
 
 static void stream_manager_delete(stream_manager_t * mgr) {
 
+	if( mgr->user_data){
+		ms_message("Destroying file %s", (char*)mgr->user_data);
+		unlink((char*)mgr->user_data);
+		ms_free(mgr->user_data);
+	}
 	if (mgr->type==MSAudio){
-		if( mgr->user_data){
-			unlink((char*)mgr->user_data);
-			// don't reset user_data, it is up to the user to free() it
-		}
-
 		audio_stream_stop(mgr->audio_stream);
 	}else{
 #if VIDEO_ENABLED
@@ -182,31 +193,50 @@ static void qos_analyzer_on_action_suggested(void *user_data, int datac, const c
 		const MSQosAnalyzer *analyzer=ms_bitrate_controller_get_qos_analyzer(mgr->video_stream->ms.rc);
 		if (analyzer->type==MSQosAnalyzerAlgorithmStateful){
 			const MSStatefulQosAnalyzer *stateful_analyzer=((const MSStatefulQosAnalyzer*)analyzer);
-			mgr->adaptive_stats.loss_estim=stateful_analyzer->network_loss_rate;
-			mgr->adaptive_stats.congestion_bw_estim=stateful_analyzer->congestion_bandwidth;
+			mgr->adaptive_stats.loss_estim=(float)stateful_analyzer->network_loss_rate;
+			mgr->adaptive_stats.congestion_bw_estim=(float)stateful_analyzer->congestion_bandwidth;
 		}
 	}
 }
 
+static void disable_plc_on_audio_stream(AudioStream *p1){
+
+	if (p1 != NULL){
+		uint32_t features_p1 = audio_stream_get_features(p1);
+		features_p1 &= ~AUDIO_STREAM_FEATURE_PLC;
+		audio_stream_set_features(p1, features_p1);
+	}
+}
+
 void start_adaptive_stream(MSFormatType type, stream_manager_t ** pmarielle, stream_manager_t ** pmargaux,
-	int payload, int initial_bitrate, int max_bw, float loss_rate, int latency, float dup_ratio) {
+	int payload, int initial_bitrate, int max_bw, float loss_rate, int latency, float dup_ratio, bool_t disable_plc) {
 	int pause_time=0;
 	PayloadType* pt;
 	MediaStream *marielle_ms,*margaux_ms;
 	OrtpNetworkSimulatorParams params={0};
+	char* recorded_file = NULL;
+	char* file = bc_tester_res(HELLO_16K_1S_FILE);
 #if VIDEO_ENABLED
-	MSWebCam * marielle_webcam=mediastreamer2_tester_get_mire_webcam(ms_web_cam_manager_get());
+	MSWebCam * marielle_webcam=mediastreamer2_tester_get_mire_webcam(ms_factory_get_web_cam_manager(_factory));
 #endif
+
 	stream_manager_t *marielle=*pmarielle=stream_manager_new(type);
 	stream_manager_t *margaux=*pmargaux=stream_manager_new(type);
 
-	char* file = bc_tester_res(HELLO_16K_1S_FILE);
-	char* recorded_file = bc_tester_file(RECORDED_16K_1S_FILE);
+	if(disable_plc){
+		disable_plc_on_audio_stream(margaux->audio_stream);
+		disable_plc_on_audio_stream(marielle->audio_stream);
+	}
+	if (!disable_plc ){
+		recorded_file = bc_tester_file(RECORDED_16K_1S_FILE);
+	}else{
+		recorded_file = bc_tester_file(RECORDED_16K_1S_NO_PLC_FILE);
+	}
 
-	marielle->user_data = recorded_file;
+	marielle->user_data = ms_strdup(recorded_file);
 	params.enabled=TRUE;
 	params.loss_rate=loss_rate;
-	params.max_bandwidth=max_bw;
+	params.max_bandwidth=(float)max_bw;
 	params.latency=latency;
 
 	if (type == MSAudio){
@@ -219,9 +249,9 @@ void start_adaptive_stream(MSFormatType type, stream_manager_t ** pmarielle, str
 
 	/* Disable avpf. */
 	pt = rtp_profile_get_payload(&rtp_profile, VP8_PAYLOAD_TYPE);
-	BC_ASSERT_PTR_NOT_NULL_FATAL(pt);
-	payload_type_unset_flag(pt, PAYLOAD_TYPE_RTCP_FEEDBACK_ENABLED);
-
+	if (BC_ASSERT_PTR_NOT_NULL(pt)) {
+		payload_type_unset_flag(pt, PAYLOAD_TYPE_RTCP_FEEDBACK_ENABLED);
+	}
 
 	media_stream_enable_adaptive_bitrate_control(marielle_ms,TRUE);
 	media_stream_set_adaptive_bitrate_algorithm(marielle_ms, MSQosAnalyzerAlgorithmStateful);
@@ -279,7 +309,7 @@ static void iterate_adaptive_stream(stream_manager_t * marielle, stream_manager_
 	}
 }
 
-static void stop_adaptive_stream(stream_manager_t *marielle, stream_manager_t *margaux){
+static void stop_adaptive_stream(stream_manager_t *marielle, stream_manager_t *margaux, bool_t destroy_files){
 	stream_manager_delete(marielle);
 	stream_manager_delete(margaux);
 }
@@ -305,10 +335,10 @@ static void event_queue_cb(MediaStream *ms, void *user_pointer) {
 						rb=rtcp_RR_get_report_block(evd->packet,0);
 					}
 
-					if (rb&&ortp_loss_rate_estimator_process_report_block(ctx->estimator,&ms->sessions.rtp_session->rtp,rb)){
-						float diff = fabs(ortp_loss_rate_estimator_get_value(ctx->estimator) - ctx->loss_rate);
-						BC_ASSERT_TRUE(diff >= 0);
-						BC_ASSERT_TRUE(diff <= 10);
+					if (rb&&ortp_loss_rate_estimator_process_report_block(ctx->estimator,ms->sessions.rtp_session,rb)){
+						float diff = (float)fabs(ortp_loss_rate_estimator_get_value(ctx->estimator) - ctx->loss_rate);
+						BC_ASSERT_GREATER(diff, 0, float, "%f");
+						BC_ASSERT_LOWER(diff, 10, float, "%f");
 					}
 				} while (rtcp_next_packet(evd->packet));
 			}
@@ -318,60 +348,113 @@ static void event_queue_cb(MediaStream *ms, void *user_pointer) {
 }
 
 /********************************** Tests are starting now ********************/
-static void packet_duplication() {
+static void packet_duplication(void) {
 	const rtp_stats_t *stats;
-	double dup_ratio;
+	float dup_ratio;
 	stream_manager_t * marielle, * margaux;
 
-	dup_ratio = 0;
-	start_adaptive_stream(MSAudio, &marielle, &margaux, SPEEX_PAYLOAD_TYPE, 32000, 0, 0, 50,dup_ratio);
+	dup_ratio = 0.0f;
+	start_adaptive_stream(MSAudio, &marielle, &margaux, SPEEX_PAYLOAD_TYPE, 32000, 0, 0.0f, 50, dup_ratio, FALSE);
 	media_stream_enable_adaptive_bitrate_control(&marielle->audio_stream->ms,FALSE);
 	iterate_adaptive_stream(marielle, margaux, 10000, NULL, 0);
 	stats=rtp_session_get_stats(margaux->video_stream->ms.sessions.rtp_session);
-	BC_ASSERT_EQUAL(stats->packet_dup_recv, dup_ratio ? stats->packet_recv / (dup_ratio+1) : 0, int, "%d");
+	BC_ASSERT_EQUAL(stats->packet_dup_recv, dup_ratio ? (uint64_t)(stats->packet_recv / (dup_ratio+1)) : 0, unsigned long long, "%llu");
 	/*in theory, cumulative loss should be the invert of duplicated count, but
 	since cumulative loss is computed only on received RTCP report and duplicated
 	count is updated on each RTP packet received, we cannot accurately compare these values*/
-	BC_ASSERT_TRUE(stats->cum_packet_loss <= -.5*stats->packet_dup_recv);
-	stop_adaptive_stream(marielle,margaux);
+	BC_ASSERT_LOWER(stats->cum_packet_loss, (int64_t)(-.5*stats->packet_dup_recv), long long, "%lld");
+	stop_adaptive_stream(marielle,margaux,TRUE);
 
-	dup_ratio = 1;
-	start_adaptive_stream(MSAudio, &marielle, &margaux, SPEEX_PAYLOAD_TYPE, 32000, 0, 0, 50,dup_ratio);
+	dup_ratio = 1.0f;
+	start_adaptive_stream(MSAudio, &marielle, &margaux, SPEEX_PAYLOAD_TYPE, 32000, 0, 0.0f, 50, dup_ratio, FALSE);
 	media_stream_enable_adaptive_bitrate_control(&marielle->audio_stream->ms,FALSE);
 	iterate_adaptive_stream(marielle, margaux, 10000, NULL, 0);
 	stats=rtp_session_get_stats(margaux->video_stream->ms.sessions.rtp_session);
-	BC_ASSERT_EQUAL(stats->packet_dup_recv, dup_ratio ? stats->packet_recv / (dup_ratio+1) : 0, int, "%d");
-	BC_ASSERT_TRUE(stats->cum_packet_loss <= -.5*stats->packet_dup_recv);
-	stop_adaptive_stream(marielle,margaux);
+	BC_ASSERT_EQUAL(stats->packet_dup_recv, dup_ratio ? (uint64_t)(stats->packet_recv / (dup_ratio+1)) : 0, unsigned long long, "%llu");
+	BC_ASSERT_LOWER(stats->cum_packet_loss, (int64_t)(-.5*stats->packet_dup_recv), long long, "%lld");
+	stop_adaptive_stream(marielle,margaux,TRUE);
 }
 
-static void upload_bandwidth_computation() {
-	bool_t supported = ms_filter_codec_supported("pcma");
+static void upload_bandwidth_computation(void) {
+
+	//bool_t supported = ms_filter_codec_supported("pcma");
+	bool_t supported = ms_factory_codec_supported(_factory, "pcma");
+
 	if( supported ) {
 		stream_manager_t * marielle, * margaux;
 		int i;
 
-		start_adaptive_stream(MSAudio, &marielle, &margaux, PCMA8_PAYLOAD_TYPE, 8000, 0, 0, 0, 0);
+		start_adaptive_stream(MSAudio, &marielle, &margaux, PCMA8_PAYLOAD_TYPE, 8000, 0, 0, 0, 0,0);
 		media_stream_enable_adaptive_bitrate_control(&marielle->audio_stream->ms,FALSE);
 
 		for (i = 0; i < 5; i++){
-			rtp_session_set_duplication_ratio(marielle->audio_stream->ms.sessions.rtp_session, i);
+			rtp_session_set_duplication_ratio(marielle->audio_stream->ms.sessions.rtp_session, (float)i);
 			iterate_adaptive_stream(marielle, margaux, 5000, NULL, 0);
 			/*since PCMA uses 80kbit/s, upload bandwidth should just be 80+80*duplication_ratio kbit/s */
-			BC_ASSERT_TRUE(fabs(rtp_session_get_send_bandwidth(marielle->audio_stream->ms.sessions.rtp_session)/1000. - 80.*(i+1)) < 5.f);
+			BC_ASSERT_LOWER((float)fabs(rtp_session_get_send_bandwidth(marielle->audio_stream->ms.sessions.rtp_session)/1000. - 80.*(i+1)), 5.f, float, "%f");
 		}
-		stop_adaptive_stream(marielle,margaux);
+		stop_adaptive_stream(marielle,margaux,TRUE);
 	}
 }
 
-static void loss_rate_estimation() {
-	bool_t supported = ms_filter_codec_supported("pcma");
+off_t fsize(const char *filename) {
+	struct stat st;
+
+	if (stat(filename, &st) == 0)
+		return st.st_size;
+
+	return -1;
+}
+
+static void loss_rate_estimation_bv16(void){
+	bool_t supported = ms_factory_codec_supported(_factory, "bv16");
+	int plc_disabled=0 ;
+	int size_with_plc = 0, size_no_plc= 0;
+	int result;
+
+	rtp_profile_set_payload(&rtp_profile,BV16_PAYLOAD_TYPE,&payload_type_bv16);
 	if( supported ) {
 		LossRateEstimatorCtx ctx;
 		stream_manager_t * marielle, * margaux;
 		int loss_rate = 15;
+		for (plc_disabled = 0; plc_disabled <=1; plc_disabled++){
+			start_adaptive_stream(MSAudio, &marielle, &margaux, BV16_PAYLOAD_TYPE, 8000, 0, (float)loss_rate, 0, 0.0f, (bool_t)plc_disabled);
+
+			ctx.estimator=ortp_loss_rate_estimator_new(120, 2500, marielle->audio_stream->ms.sessions.rtp_session);
+			ctx.q = ortp_ev_queue_new();
+			rtp_session_register_event_queue(marielle->audio_stream->ms.sessions.rtp_session, ctx.q);
+			ctx.loss_rate = loss_rate;
+
+			/*loss rate should be the initial one*/
+			wait_for_until_with_parse_events(&marielle->audio_stream->ms, &margaux->audio_stream->ms, &loss_rate, 100, 10000, event_queue_cb,&ctx,NULL,NULL);
+
+			stop_adaptive_stream(marielle,margaux,FALSE);
+			ortp_loss_rate_estimator_destroy(ctx.estimator);
+			ortp_ev_queue_destroy(ctx.q);
+		}
+		/*file without plc must be approx 15% smaller in size than with plc */
+		size_with_plc= fsize(bc_tester_file(RECORDED_16K_1S_FILE));
+		size_no_plc = fsize(bc_tester_file(RECORDED_16K_1S_NO_PLC_FILE));
+		result = (size_with_plc*loss_rate/100 + size_no_plc) ;
+
+		BC_ASSERT_GREATER(result, size_with_plc, int, "%d");
+		// if test worked, remove files
+		if (result >= size_with_plc) {
+			unlink(RECORDED_16K_1S_FILE);
+			unlink(RECORDED_16K_1S_NO_PLC_FILE);
+		}
+	}
+}
+
+static void loss_rate_estimation(void) {
+
+	bool_t supported = ms_factory_codec_supported(_factory, "pcma");
+	if( supported ) {
+		LossRateEstimatorCtx ctx;
+		stream_manager_t * marielle, * margaux;
+		int loss_rate = 15;
+		start_adaptive_stream(MSAudio, &marielle, &margaux, PCMA8_PAYLOAD_TYPE, 8000, 0, (float)loss_rate, 0, 0.0f, FALSE);
 
-		start_adaptive_stream(MSAudio, &marielle, &margaux, PCMA8_PAYLOAD_TYPE, 8000, 0, loss_rate, 0, 0);
 		ctx.estimator=ortp_loss_rate_estimator_new(120, 2500, marielle->audio_stream->ms.sessions.rtp_session);
 		ctx.q = ortp_ev_queue_new();
 		rtp_session_register_event_queue(marielle->audio_stream->ms.sessions.rtp_session, ctx.q);
@@ -384,50 +467,51 @@ static void loss_rate_estimation() {
 		rtp_session_set_duplication_ratio(marielle->audio_stream->ms.sessions.rtp_session, 10);
 		wait_for_until_with_parse_events(&marielle->audio_stream->ms, &margaux->audio_stream->ms, &loss_rate, 100, 10000, event_queue_cb,&ctx,NULL,NULL);
 
-		stop_adaptive_stream(marielle,margaux);
+		stop_adaptive_stream(marielle,margaux,TRUE);
 		ortp_loss_rate_estimator_destroy(ctx.estimator);
 		ortp_ev_queue_destroy(ctx.q);
 	}
 }
 
 void upload_bitrate(const char* codec, int payload, int target_bw, int expect_bw) {
-	bool_t supported = ms_filter_codec_supported(codec);
+	//bool_t supported = ms_filter_codec_supported("pcma");
+	bool_t supported = ms_factory_codec_supported(_factory, codec);
 	if( supported ) {
 		float upload_bw;
 		stream_manager_t * marielle, * margaux;
 
-		start_adaptive_stream(MSAudio, &marielle, &margaux, payload, target_bw*1000, target_bw*1000, 0, 50,0);
+		start_adaptive_stream(MSAudio, &marielle, &margaux, payload, target_bw*1000, target_bw*1000, 0, 50,0,0);
 		//these tests check that encoders stick to the guidelines, so we must use NOT
 		//the adaptive algorithm which would modify these guidelines
 		media_stream_enable_adaptive_bitrate_control(&marielle->audio_stream->ms,FALSE);
 		iterate_adaptive_stream(marielle, margaux, 15000, NULL, 0);
 		upload_bw=media_stream_get_up_bw(&marielle->audio_stream->ms) / 1000;
-		BC_ASSERT_TRUE(upload_bw >= expect_bw-2);
-		BC_ASSERT_TRUE(upload_bw <= expect_bw+2);
-		stop_adaptive_stream(marielle,margaux);
+		BC_ASSERT_GREATER(upload_bw, (float)(expect_bw-2), float, "%f");
+		BC_ASSERT_LOWER(upload_bw, (float)(expect_bw+2), float, "%f");
+		stop_adaptive_stream(marielle,margaux,TRUE);
 	}
 }
 
-static void upload_bitrate_pcma_3g() {
+static void upload_bitrate_pcma_3g(void) {
 	// pcma codec bitrate is always 64 kbits, only ptime can change from 20ms to 100ms.
 	// ptime=20  ms -> network bitrate=80 kbits/s
 	// ptime=100 ms -> network bitrate=67 kbits/s
 	upload_bitrate("pcma", PCMA8_PAYLOAD_TYPE, THIRDGENERATION_BW, 80);
 }
 
-static void upload_bitrate_speex_low()  {
+static void upload_bitrate_speex_low(void)  {
 	// speex codec bitrate can vary from 16 kbits/s to 42 kbits/s
 	// bitrate=42 kbits/s ptime=20  ms -> network bitrate=58 kbits/s
 	// bitrate=16 kbits/s ptime=100 ms -> network bitrate=19 kbits/s
 	upload_bitrate("speex", SPEEX_PAYLOAD_TYPE, 25, 25);
 }
 
-static void upload_bitrate_speex_3g()  {
+static void upload_bitrate_speex_3g(void) {
 	upload_bitrate("speex", SPEEX_PAYLOAD_TYPE, THIRDGENERATION_BW, 59);
 }
 
 
-static void upload_bitrate_opus_edge() {
+static void upload_bitrate_opus_edge(void) {
 	// opus codec bitrate can vary from 6 kbits/s to 184 kbits/s
 	// bitrate=6   kbits/s and ptime=100  ms -> network bitrate=  9 kbits/s
 	// bitrate=184 kbits/s and ptime=20   ms -> network bitrate=200 kbits/s
@@ -437,7 +521,7 @@ static void upload_bitrate_opus_edge() {
 	// is not sufficient to match the guidelines without adaptive algorithm.
 }
 
-static void upload_bitrate_opus_3g() {
+static void upload_bitrate_opus_3g(void) {
 	upload_bitrate("opus", OPUS_PAYLOAD_TYPE, THIRDGENERATION_BW, 200);
 }
 
@@ -448,11 +532,11 @@ void adaptive_video(int max_bw, int exp_min_bw, int exp_max_bw, int loss_rate, i
 		stream_manager_t * marielle, * margaux;
 		start_adaptive_stream(MSVideo, &marielle, &margaux, VP8_PAYLOAD_TYPE, 300*1000, max_bw*1000, loss_rate, 50,0);
 		iterate_adaptive_stream(marielle, margaux, 100000, &marielle->rtcp_count, 7);
-		BC_ASSERT_TRUE(marielle->adaptive_stats.loss_estim >= exp_min_loss);
-		BC_ASSERT_TRUE(marielle->adaptive_stats.loss_estim <= exp_max_loss);
-		BC_ASSERT_TRUE(marielle->adaptive_stats.congestion_bw_estim >= exp_min_bw);
-		BC_ASSERT_TRUE(marielle->adaptive_stats.congestion_bw_estim <= exp_max_bw);
-		stop_adaptive_stream(marielle,margaux);
+		BC_ASSERT_GREATER(marielle->adaptive_stats.loss_estim, exp_min_loss, int, "%d");
+		BC_ASSERT_LOWER(marielle->adaptive_stats.loss_estim, exp_max_loss, int, "%d");
+		BC_ASSERT_GREATER(marielle->adaptive_stats.congestion_bw_estim, exp_min_bw, int, "%d");
+		BC_ASSERT_LOWER(marielle->adaptive_stats.congestion_bw_estim, exp_max_bw, int, "%d");
+		stop_adaptive_stream(marielle,margaux,TRUE);
 	}
 }
 
@@ -476,6 +560,7 @@ static test_t tests[] = {
 	{ "Packet duplication", packet_duplication},
 	{ "Upload bandwidth computation", upload_bandwidth_computation },
 	{ "Loss rate estimation", loss_rate_estimation },
+	{ "Loss rate estimationBV16", loss_rate_estimation_bv16 },
 
 	{ "Upload bitrate [pcma] - 3g", upload_bitrate_pcma_3g },
 	{ "Upload bitrate [speex] - low", upload_bitrate_speex_low },
