@@ -26,6 +26,12 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "mediastreamer2_tester.h"
 #include "mediastreamer2_tester_private.h"
 
+#ifdef _MSC_VER
+#pragma warning(disable : 4996)
+#else
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
+
 static int sound_card_tester_before_all(void) {
 	ms_init();
 	ms_filter_enable_statistics(TRUE);
@@ -62,7 +68,7 @@ static void dtmfgen_soundwrite(void) {
 
 	ms_filter_reset_statistics();
 	ms_tester_create_ticker();
-	ms_tester_create_filters(filter_mask);
+	ms_tester_create_filters(filter_mask, ms_factory_get_fallback());
 	ms_filter_call_method(ms_tester_voidsource, MS_FILTER_SET_SAMPLE_RATE, &sample_rate);
 	ms_filter_call_method(ms_tester_voidsource, MS_FILTER_SET_NCHANNELS, &nchannels);
 	ms_filter_call_method(ms_tester_voidsource, MS_VOID_SOURCE_SEND_SILENCE, &send_silence);
@@ -75,7 +81,7 @@ static void dtmfgen_soundwrite(void) {
 		need_resampler = TRUE;
 	}
 	if (need_resampler == TRUE) {
-		ms_tester_create_filters(FILTER_MASK_RESAMPLER);
+		ms_tester_create_filters(FILTER_MASK_RESAMPLER, ms_factory_get_fallback());
 		configure_resampler(ms_tester_resampler, ms_tester_dtmfgen, ms_tester_soundwrite);
 	}
 	ms_connection_helper_start(&h);
@@ -133,7 +139,7 @@ static void fileplay_soundwrite(const char *filename) {
 
 	ms_filter_reset_statistics();
 	ms_tester_create_ticker();
-	ms_tester_create_filters(filter_mask);
+	ms_tester_create_filters(filter_mask, ms_factory_get_fallback());
 	ms_filter_add_notify_callback(ms_tester_fileplay, fileplay_eof, &done,TRUE);
 	ms_filter_call_method_noarg(ms_tester_fileplay, MS_FILE_PLAYER_CLOSE);
 	ms_filter_call_method(ms_tester_fileplay, MS_FILE_PLAYER_OPEN, (void *)filename);
@@ -150,7 +156,7 @@ static void fileplay_soundwrite(const char *filename) {
 		if (nchannels != soundwrite_nchannels) need_resampler = TRUE;
 	}
 	if (need_resampler == TRUE) {
-		ms_tester_create_filters(FILTER_MASK_RESAMPLER);
+		ms_tester_create_filters(FILTER_MASK_RESAMPLER, ms_factory_get_fallback());
 		configure_resampler(ms_tester_resampler, ms_tester_fileplay, ms_tester_soundwrite);
 	}
 	ms_filter_call_method_noarg(ms_tester_fileplay, MS_FILE_PLAYER_START);
@@ -231,7 +237,7 @@ static void soundread_soundwrite(void) {
 
 	ms_filter_reset_statistics();
 	ms_tester_create_ticker();
-	ms_tester_create_filters(filter_mask);
+	ms_tester_create_filters(filter_mask, ms_factory_get_fallback());
 	ms_filter_call_method(ms_tester_soundread, MS_FILTER_GET_SAMPLE_RATE, &sample_rate);
 	ms_filter_call_method(ms_tester_soundread, MS_FILTER_GET_NCHANNELS, &nchannels);
 	if (ms_filter_call_method(ms_tester_soundwrite, MS_FILTER_SET_SAMPLE_RATE, &sample_rate) != 0) {
@@ -245,7 +251,7 @@ static void soundread_soundwrite(void) {
 		if (nchannels != soundwrite_nchannels) need_resampler = TRUE;
 	}
 	if (need_resampler == TRUE) {
-		ms_tester_create_filters(FILTER_MASK_RESAMPLER);
+		ms_tester_create_filters(FILTER_MASK_RESAMPLER, ms_factory_get_fallback());
 		configure_resampler(ms_tester_resampler, ms_tester_soundread, ms_tester_soundwrite);
 	}
 	ms_connection_helper_start(&h);
@@ -273,6 +279,107 @@ static void soundread_soundwrite(void) {
 	ms_tester_destroy_ticker();
 }
 
+static void fileplay_bv16enc_bv16dec_soundwrite(void) {
+	if (ms_factory_codec_supported(ms_factory_get_fallback(), "bv16")) {
+		MSConnectionHelper h;
+		MSFilter *read_resampler = NULL, *write_resampler = NULL;
+		bool_t need_read_resampler = FALSE, need_write_resampler = TRUE;
+		unsigned int filter_mask = FILTER_MASK_FILEPLAY  | FILTER_MASK_ENCODER | FILTER_MASK_DECODER | FILTER_MASK_SOUNDWRITE;
+		int sample_rate = 8000;
+		int nchannels = 1;
+		int done = FALSE;
+		const char* filepath = ARPEGGIO_8000_MONO_FILE_NAME;
+
+		char* filename = bc_tester_res(filepath);
+
+		int elapsed = 0;
+
+		ms_filter_reset_statistics();
+		ms_tester_create_ticker();
+		ms_tester_codec_mime = "bv16";
+		ms_tester_create_filters(filter_mask, ms_factory_get_fallback());
+
+
+		//file
+		ms_filter_add_notify_callback(ms_tester_fileplay, fileplay_eof, &done,TRUE);
+		ms_filter_call_method_noarg(ms_tester_fileplay, MS_FILE_PLAYER_CLOSE);
+
+		ms_filter_call_method(ms_tester_fileplay, MS_FILE_PLAYER_OPEN, (void *)filename);
+		ms_filter_call_method_noarg(ms_tester_fileplay, MS_FILE_PLAYER_START);
+		ms_filter_call_method(ms_tester_fileplay, MS_FILTER_GET_SAMPLE_RATE, &sample_rate);
+		ms_filter_call_method(ms_tester_fileplay, MS_FILTER_GET_NCHANNELS, &nchannels);
+
+		ms_filter_call_method(ms_tester_encoder, MS_FILTER_GET_BITRATE, &sample_rate);
+		ms_filter_call_method(ms_tester_decoder, MS_FILTER_SET_BITRATE, &sample_rate);
+
+
+		if (need_read_resampler == TRUE) {
+			ms_tester_create_filters(FILTER_MASK_RESAMPLER, ms_factory_get_fallback());
+			configure_resampler(ms_tester_resampler, ms_tester_fileplay, ms_tester_encoder);
+		}
+
+
+		ms_connection_helper_start(&h);
+		ms_connection_helper_link(&h, ms_tester_fileplay, -1, 0);
+		if (need_read_resampler == TRUE) {
+			ms_connection_helper_link(&h, ms_tester_resampler, 0, 0);
+		}
+
+		if (ms_filter_call_method(ms_tester_soundwrite, MS_FILTER_SET_BITRATE, &sample_rate) != 0) {
+			int soundwrite_sample_rate = 8000;
+			ms_filter_call_method(ms_tester_soundwrite, MS_FILTER_GET_BITRATE, &soundwrite_sample_rate);
+			if (sample_rate != soundwrite_sample_rate) need_write_resampler = TRUE;
+		}
+		if (ms_filter_call_method(ms_tester_soundwrite, MS_FILTER_SET_NCHANNELS, &nchannels) != 0) {
+			int soundwrite_nchannels = 1;
+			ms_filter_call_method(ms_tester_soundwrite, MS_FILTER_GET_NCHANNELS, &soundwrite_nchannels);
+			if (nchannels != soundwrite_nchannels) need_write_resampler = TRUE;
+		}
+		if (need_write_resampler == TRUE) {
+			ms_tester_create_filter(&write_resampler, MS_RESAMPLE_ID, ms_factory_get_fallback());
+			configure_resampler(write_resampler, ms_tester_decoder, ms_tester_soundwrite);
+		}
+
+		ms_connection_helper_link(&h, ms_tester_encoder, 0, 0);
+		ms_connection_helper_link(&h, ms_tester_decoder, 0, 0);
+		if (need_write_resampler == TRUE) {
+			ms_connection_helper_link(&h, write_resampler, 0, 0);
+		}
+		ms_connection_helper_link(&h, ms_tester_soundwrite, 0, -1);
+		ms_ticker_attach(ms_tester_ticker, ms_tester_fileplay);
+
+
+		while (done != TRUE && elapsed < 6000) {
+			elapsed++;
+			ms_usleep(10000);
+		}
+
+		ms_filter_call_method_noarg(ms_tester_fileplay, MS_FILE_PLAYER_CLOSE);
+		ms_ticker_detach(ms_tester_ticker, ms_tester_fileplay);
+		ms_connection_helper_start(&h);
+		ms_connection_helper_unlink(&h, ms_tester_fileplay, -1, 0);
+		if (need_read_resampler == TRUE) {
+			ms_connection_helper_unlink(&h, ms_tester_resampler, 0, 0);
+		}
+		ms_connection_helper_unlink(&h, ms_tester_encoder, 0, 0);
+		ms_connection_helper_unlink(&h, ms_tester_decoder, 0, 0);
+		if (need_write_resampler == TRUE) {
+			ms_connection_helper_unlink(&h, write_resampler, 0, 0);
+		}
+		ms_connection_helper_unlink(&h, ms_tester_soundwrite, 0, -1);
+		if (need_read_resampler == TRUE) {
+			ms_tester_destroy_filter(&read_resampler);
+		}
+		if (need_write_resampler == TRUE) {
+			ms_tester_destroy_filter(&write_resampler);
+		}
+		ms_filter_log_statistics();
+		ms_tester_destroy_filters(filter_mask);
+		ms_tester_destroy_ticker();
+		free(filename);
+	}
+}
+
 static void soundread_speexenc_speexdec_soundwrite(void) {
 	MSConnectionHelper h;
 	MSFilter *read_resampler = NULL, *write_resampler = NULL;
@@ -284,7 +391,7 @@ static void soundread_speexenc_speexdec_soundwrite(void) {
 	ms_filter_reset_statistics();
 	ms_tester_create_ticker();
 	ms_tester_codec_mime = "speex";
-	ms_tester_create_filters(filter_mask);
+	ms_tester_create_filters(filter_mask, ms_factory_get_fallback());
 	ms_filter_call_method(ms_tester_encoder, MS_FILTER_GET_BITRATE, &sample_rate);
 	ms_filter_call_method(ms_tester_decoder, MS_FILTER_SET_BITRATE, &sample_rate);
 	if (ms_filter_call_method(ms_tester_soundread, MS_FILTER_SET_BITRATE, &sample_rate) != 0) {
@@ -298,7 +405,7 @@ static void soundread_speexenc_speexdec_soundwrite(void) {
 		if (nchannels != soundread_nchannels) need_read_resampler = TRUE;
 	}
 	if (need_read_resampler == TRUE) {
-		ms_tester_create_filter(&read_resampler, MS_RESAMPLE_ID);
+		ms_tester_create_filter(&read_resampler, MS_RESAMPLE_ID, ms_factory_get_fallback());
 		configure_resampler(read_resampler, ms_tester_soundread, ms_tester_encoder);
 	}
 	if (ms_filter_call_method(ms_tester_soundwrite, MS_FILTER_SET_BITRATE, &sample_rate) != 0) {
@@ -312,7 +419,7 @@ static void soundread_speexenc_speexdec_soundwrite(void) {
 		if (nchannels != soundwrite_nchannels) need_write_resampler = TRUE;
 	}
 	if (need_write_resampler == TRUE) {
-		ms_tester_create_filter(&write_resampler, MS_RESAMPLE_ID);
+		ms_tester_create_filter(&write_resampler, MS_RESAMPLE_ID, ms_factory_get_fallback());
 		configure_resampler(write_resampler, ms_tester_decoder, ms_tester_soundwrite);
 	}
 	ms_connection_helper_start(&h);
@@ -366,7 +473,7 @@ static void soundread_filerec_fileplay_soundwrite(void) {
 
 	ms_filter_reset_statistics();
 	ms_tester_create_ticker();
-	ms_tester_create_filters(filter_mask);
+	ms_tester_create_filters(filter_mask, ms_factory_get_fallback());
 
 	// Write audio capture to a file
 	ms_filter_call_method(ms_tester_soundread, MS_FILTER_GET_SAMPLE_RATE, &capture_sample_rate);
@@ -393,7 +500,7 @@ static void soundread_filerec_fileplay_soundwrite(void) {
 	ms_filter_call_method(ms_tester_soundwrite, MS_FILTER_GET_SAMPLE_RATE, &playback_sample_rate);
 	ms_filter_call_method(ms_tester_soundwrite, MS_FILTER_GET_NCHANNELS, &playback_nchannels);
 	if ((capture_sample_rate != playback_sample_rate) || (capture_nchannels != playback_nchannels)) {
-		ms_tester_create_filter(&ms_tester_resampler, MS_RESAMPLE_ID);
+		ms_tester_create_filter(&ms_tester_resampler, MS_RESAMPLE_ID, ms_factory_get_fallback());
 	}
 	ms_filter_call_method_noarg(ms_tester_fileplay, MS_FILE_PLAYER_CLOSE);
 	ms_filter_call_method(ms_tester_fileplay, MS_FILE_PLAYER_OPEN, writable_filename);
@@ -441,7 +548,8 @@ test_t sound_card_tests[] = {
 	{ "fileplay-soundwrite-8000-mono", fileplay_soundwrite_8000_mono },
 	{ "soundread-soundwrite", soundread_soundwrite },
 	{ "soundread-speexenc-speexdec-soundwrite", soundread_speexenc_speexdec_soundwrite },
-	{ "soundread-filerec-fileplay-soundwrite", soundread_filerec_fileplay_soundwrite }
+	{ "soundread-filerec-fileplay-soundwrite", soundread_filerec_fileplay_soundwrite },
+	{ "fileplay-bv16enc-bv16dec-soundwrite", fileplay_bv16enc_bv16dec_soundwrite}
 };
 
 test_suite_t sound_card_test_suite = {
