@@ -40,13 +40,13 @@ static void log_handler(int lev, const char *fmt, va_list args) {
 	va_end(cap);
 #endif
 	if (log_file){
-		ortp_logv_out(lev, fmt, args);
+		ortp_logv_out(ORTP_LOG_DOMAIN, lev, fmt, args);
 	}
 }
 
 void mediastreamer2_tester_init(void(*ftester_printf)(int level, const char *fmt, va_list args)) {
 	if (ftester_printf == NULL) ftester_printf = log_handler;
-	bc_tester_init(ftester_printf, ORTP_MESSAGE, ORTP_ERROR);
+	bc_tester_init(ftester_printf, ORTP_MESSAGE, ORTP_ERROR, "sounds");
 
 	bc_tester_add_suite(&basic_audio_test_suite);
 	bc_tester_add_suite(&sound_card_test_suite);
@@ -61,6 +61,9 @@ void mediastreamer2_tester_init(void(*ftester_printf)(int level, const char *fmt
 	bc_tester_add_suite(&neon_test_suite);
 #endif
 	bc_tester_add_suite(&text_stream_test_suite);
+#ifdef HAVE_PCAP
+	bc_tester_add_suite(&codec_impl_test_suite);
+#endif
 }
 
 void mediastreamer2_tester_uninit(void) {
@@ -106,7 +109,7 @@ int main (int argc, char *argv[]) {
 
 	// this allows to launch tester from outside of tester directory
 	if (strstr(argv[0], ".libs")) {
-		int prefix_length = strstr(argv[0], ".libs") - argv[0] + 1;
+		long prefix_length = strstr(argv[0], ".libs") - argv[0] + 1;
 		char *prefix = ms_strdup_printf("%s%.*s", argv[0][0] == '/' ? "" : "./", prefix_length, argv[0]);
 		ms_warning("Resource prefix set to %s", prefix);
 		bc_tester_set_resource_dir_prefix(prefix);
@@ -115,9 +118,9 @@ int main (int argc, char *argv[]) {
 	}
 	for(i = 1; i < argc; ++i) {
 		if (strcmp(argv[i], "--verbose") == 0) {
-			ortp_set_log_level_mask(ORTP_MESSAGE|ORTP_WARNING|ORTP_ERROR|ORTP_FATAL);
+			ortp_set_log_level_mask(ORTP_LOG_DOMAIN, ORTP_MESSAGE|ORTP_WARNING|ORTP_ERROR|ORTP_FATAL);
 		} else if (strcmp(argv[i], "--silent") == 0) {
-			ortp_set_log_level_mask(ORTP_FATAL);
+			ortp_set_log_level_mask(ORTP_LOG_DOMAIN, ORTP_FATAL);
 		} else if (strcmp(argv[i],"--log-file")==0){
 			CHECK_ARG("--log-file", ++i, argc);
 			if (mediastreamer2_tester_set_log_file(argv[i]) < 0) return -2;
