@@ -29,21 +29,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #define unlink _unlink
 #endif
 
-#if WINAPI_FAMILY_PHONE_APP
-	#define SOUND_FILE_PATH		"Assets\\"
-#else
-	#if __QNX__
-		#define SOUND_FILE_PATH		"./app/native/assets/"
-	#else
-		#define SOUND_FILE_PATH		"./"
-	#endif
-#endif
-#if __QNX__
-	#define WRITE_FILE_PATH	"./tmp/"
-#else
-	#define WRITE_FILE_PATH "."
-#endif
-
 
 #ifdef __cplusplus
 extern "C"
@@ -90,8 +75,8 @@ extern unsigned char ms_tester_tone_detected;
 
 void ms_tester_create_ticker(void);
 void ms_tester_destroy_ticker(void);
-void ms_tester_create_filter(MSFilter **filter, MSFilterId id);
-void ms_tester_create_filters(unsigned int filter_mask);
+void ms_tester_create_filter(MSFilter **filter, MSFilterId id, MSFactory *f);
+void ms_tester_create_filters(unsigned int filter_mask, MSFactory *f);
 void ms_tester_destroy_filter(MSFilter **filter);
 void ms_tester_destroy_filters(unsigned int filter_mask);
 void ms_tester_tone_generation_loop(void);
